@@ -5,8 +5,9 @@ from pyro.infer.autoguide.guides import (AutoCallable, AutoContinuous, AutoDelta
                                          AutoDiscreteParallel, AutoGuide, AutoGuideList, AutoIAFNormal,
                                          AutoLaplaceApproximation, AutoLowRankMultivariateNormal,
                                          AutoMultivariateNormal, AutoNormal, AutoNormalizingFlow)
+from pyro.infer.autoguide.initialization import (init_to_feasible, init_to_generated, init_to_mean, init_to_median,
+                                                 init_to_sample, init_to_uniform, init_to_value)
 from pyro.infer.autoguide.utils import mean_field_entropy
-from pyro.infer.autoguide.initialization import init_to_feasible, init_to_mean, init_to_median, init_to_sample
 
 __all__ = [
     'AutoCallable',
@@ -23,8 +24,11 @@ __all__ = [
     'AutoNormal',
     'AutoNormalizingFlow',
     'init_to_feasible',
+    'init_to_generated',
     'init_to_mean',
     'init_to_median',
     'init_to_sample',
+    'init_to_uniform',
+    'init_to_value',
     'mean_field_entropy',
 ]
