@@ -1,11 +1,14 @@
 # Copyright (c) 2017-2019 Uber Technologies, Inc.
 # SPDX-License-Identifier: Apache-2.0
 
+from pyro.infer.mcmc.adaptation import ArrowheadMassMatrix, BlockMassMatrix
 from pyro.infer.mcmc.hmc import HMC
 from pyro.infer.mcmc.api import MCMC
 from pyro.infer.mcmc.nuts import NUTS
 
 __all__ = [
+    "ArrowheadMassMatrix",
+    "BlockMassMatrix",
     "HMC",
     "MCMC",
     "NUTS",
