@@ -1,24 +1,31 @@
+# Copyright Contributors to the Pyro project.
 # Copyright (c) 2017-2019 Uber Technologies, Inc.
 # SPDX-License-Identifier: Apache-2.0
 
+from .conjugate import ConjugateReparam
 from .discrete_cosine import DiscreteCosineReparam
+from .haar import HaarReparam
 from .hmm import LinearHMMReparam
 from .loc_scale import LocScaleReparam
 from .neutra import NeuTraReparam
-from .conjugate import ConjugateReparam
+from .split import SplitReparam
 from .stable import LatentStableReparam, StableReparam, SymmetricStableReparam
 from .studentt import StudentTReparam
 from .transform import TransformReparam
+from .unit_jacobian import UnitJacobianReparam
 
 __all__ = [
     "ConjugateReparam",
     "DiscreteCosineReparam",
+    "HaarReparam",
     "LatentStableReparam",
     "LinearHMMReparam",
     "LocScaleReparam",
     "NeuTraReparam",
+    "SplitReparam",
     "StableReparam",
     "StudentTReparam",
     "SymmetricStableReparam",
     "TransformReparam",
+    "UnitJacobianReparam",
 ]
